import React, { useState } from 'react';
import { Input, LoadingSpinner } from '../../components';
import { Accordion, AccordionBody, AccordionHead, AccordionItem } from '../../components/accordion';
import { TextEditor } from '../../components/textEditor';
import { EditorState } from 'lexical';

type Props = {
  activeTemplate: string;
};

export function ReportingForm(props: Props) {
  const [loading, setLoading] = useState(false);
  const [editorState, setEditorState] = useState<EditorState | undefined>(undefined);

  return loading ? (
    <LoadingSpinner />
  ) : (
    props.activeTemplate && (
      <div>
        <span className="text-lg text-secondary-600 font-bold mb-4">Report ID: {props.activeTemplate}</span>
        <Accordion defaultOpenAll={true} className="border-t divide-y">
          <AccordionItem className="pt-2 pb-4">
            <AccordionHead showArrow={false}>
              <span className="font-semibold">Recipient(s)</span>
            </AccordionHead>
            <AccordionBody>
              <div>
                <input className="border" />
              </div>
            </AccordionBody>
          </AccordionItem>
          <AccordionItem className="pt-2 pb-4">
            <AccordionHead showArrow={false}>
              <span className="font-semibold">Repeat</span>
            </AccordionHead>
            <AccordionBody>
              <Input label="Frequency" type="dropdown" value={'Daily'} onChange={() => {}} options={['Daily', 'Weekly']} className="mb-1" />
            </AccordionBody>
          </AccordionItem>
          <AccordionItem className="pt-2 pb-4">
            <AccordionHead showArrow={false}>
              <span className="font-semibold">Email template</span>
            </AccordionHead>
            <AccordionBody>
              <TextEditor
                editorState={editorState}
                setEditorState={setEditorState}
                showToolbar={true}
                placeholder="Start typing your report template..."
              />
            </AccordionBody>
          </AccordionItem>
        </Accordion>
      </div>
    )
  );
}
