export type InputNodeTypeTypes = string | number | boolean;
export type InputNodeType = 'string' | 'float' | 'int' | 'bool' | 'date' | 'time' | 'datetime' | 'duration';
export type InputNodeDimension = 'categorical' | 'numerical' | 'temporal' | 'spatial';

export enum NumberFilterTypes {
  EQUAL = '==',
  NOT_EQUAL = '!=',
  GREATER_THAN = '>',
  LESS_THAN = '<',
  GREATER_THAN_EQUAL = '>=',
  LESS_THAN_EQUAL = '<=',
}

export enum NumberAggregationTypes {
  AVG = 'Avg',
  COUNT = 'Count',
  MAX = 'Max',
  MIN = 'Min',
  SUM = 'Sum',
}

export enum NumberFunctionTypes {
  ADD = '+',
  SUBTRACT = '-',
  MULTIPLY = '*',
  DIVIDE = '/',
  // MODULO = '%',
  // POWER = '^',
  // SQRT = 'Sqrt',
  // ABS = 'Abs',
  // LOG = 'Log',
  // EXP = 'Exp',
  // ROUND = 'Round',
  // CEIL = 'Ceil',
  // FLOOR = 'Floor',
}

export enum StringFunctionTypes {
  LOWER = 'Lower',
  UPPER = 'Upper',
  // CONCAT = 'Concat',
  // SUBSTRING = 'Substring',
  // TRIM = 'Trim',
}

export enum StringFilterTypes {
  EQUAL = '==',
  NOT_EQUAL = '!=',
  LIKE = 'Like',
  EMPTY = 'IsEmpty',
  // NOT_LIKE = 'Not Like',
  // IN = 'In',
  // NOT_IN = 'Not In',
}

export enum LogicFunctionTypes {
  AND = 'And',
  OR = 'Or',
  NOT = 'Not',
}

// Logic
export type AndLogicStatement = ['And', AnyStatement, AnyStatement];
export type OrLogicStatement = ['Or', AnyStatement, AnyStatement];
export type NotLogicStatement = ['Not', AnyStatement];
export type LogicStatement = AndLogicStatement | OrLogicStatement | NotLogicStatement;

// Numbers
export type PlusStatement = ['+', AnyStatement, AnyStatement];
export type MinusStatement = ['-', AnyStatement, AnyStatement];
export type MultiplyStatement = ['*', AnyStatement, AnyStatement];
export type DivideStatement = ['/', AnyStatement, AnyStatement];
export type ModuloStatement = ['%', AnyStatement, AnyStatement];
export type PowerStatement = ['^', AnyStatement, AnyStatement];
export type SqrtStatement = ['Sqrt', AnyStatement];
export type AbsStatement = ['Abs', AnyStatement];
export type LogStatement = ['Log', AnyStatement];
export type ExpStatement = ['Exp', AnyStatement];
export type AllMathStatement =
  | PlusStatement
  | MinusStatement
  | MultiplyStatement
  | DivideStatement
  | ModuloStatement
  | PowerStatement
  | SqrtStatement
  | AbsStatement
  | LogStatement
  | ExpStatement;

// Comparisons
export type EqualStatement = ['==', AnyStatement, AnyStatement];
export type NotEqualStatement = ['!=', AnyStatement, AnyStatement];
export type GreaterThanStatement = ['>', AnyStatement, AnyStatement];
export type LessThanStatement = ['<', AnyStatement, AnyStatement];
export type GreaterThanEqualStatement = ['>=', AnyStatement, AnyStatement];
export type LessThanEqualStatement = ['<=', AnyStatement, AnyStatement];
export type AllComparisonStatement =
  | EqualStatement
  | NotEqualStatement
  | GreaterThanStatement
  | LessThanStatement
  | GreaterThanEqualStatement
  | LessThanEqualStatement;

// Strings
export type ConcatStatement = ['Concat', AnyStatement, AnyStatement];
export type LowerStatement = ['Lower', AnyStatement];
export type UpperStatement = ['Upper', AnyStatement];
export type SubstringStatement = ['Substring', AnyStatement, AnyStatement, AnyStatement];
export type TrimStatement = ['Trim', AnyStatement];
export type EmptyStatement = [StringFilterTypes.EMPTY, AnyStatement];
export type AllStringStatement = ConcatStatement | LowerStatement | UpperStatement | SubstringStatement | TrimStatement | EmptyStatement;

// Dates
export type DateStatement = ['Date', AnyStatement];
export type YearStatement = ['Year', AnyStatement];
export type MonthStatement = ['Month', AnyStatement];
export type DayStatement = ['Day', AnyStatement];
export type HourStatement = ['Hour', AnyStatement];
export type MinuteStatement = ['Minute', AnyStatement];
export type SecondStatement = ['Second', AnyStatement];
export type AllDateStatement =
  | DateStatement
  | YearStatement
  | MonthStatement
  | DayStatement
  | HourStatement
  | MinuteStatement
  | SecondStatement;

// Aggregations
export type AvgStatement = ['Avg', AnyStatement];
export type CountStatement = ['Count', AnyStatement];
export type MaxStatement = ['Max', AnyStatement];
export type MinStatement = ['Min', AnyStatement];
export type SumStatement = ['Sum', AnyStatement];
export type RoundStatement = ['Round', AnyStatement];
export type CeilStatement = ['Ceil', AnyStatement];
export type FloorStatement = ['Floor', AnyStatement];
export type AllAggregationStatement =
  | AvgStatement
  | CountStatement
  | MaxStatement
  | MinStatement
  | SumStatement
  | RoundStatement
  | CeilStatement
  | FloorStatement;

// Filters
export type EqualFilterStatement = ['==', AnyStatement, AnyStatement];
export type NotEqualFilterStatement = ['!=', AnyStatement, AnyStatement];
export type LikeFilterStatement = ['Like', AnyStatement, AnyStatement];
export type NotLikeFilterStatement = ['Not Like', AnyStatement, AnyStatement];
export type InFilterStatement = ['In', AnyStatement, AnyStatement];
export type NotInFilterStatement = ['Not In', AnyStatement, AnyStatement];
export type FilterStatement =
  | EqualFilterStatement
  | NotEqualFilterStatement
  | LikeFilterStatement
  | NotLikeFilterStatement
  | InFilterStatement
  | NotInFilterStatement;

// Regular
type WithPrefix<T extends string> = `${T}${string}`;

export type NumberStatement = number;
export type StringStatement = string;
export type BooleanStatement = boolean;
export type DateStringStatement = string;
export type ReferenceStatement = WithPrefix<'@'>;
export type RegularStatement = NumberStatement | StringStatement | BooleanStatement | DateStringStatement;
export type AllRegularStatement = RegularStatement | ReferenceStatement;

export type AllLogicStatement =
  | LogicStatement
  | AllMathStatement
  | AllComparisonStatement
  | AllStringStatement
  | AllDateStatement
  | AllAggregationStatement
  | FilterStatement;

export type AnyStatement = AllLogicStatement | AllRegularStatement;

export interface OutputNode {
  name: string;
  type: InputNodeType;
}

export interface InputNode {
  name: string;
  type: InputNodeType;
  default: number | string | boolean;
}

export interface GeneralDescription<T> {
  name: string;
  type: T;
  description: string;
  numExtraInputs: number;
  input: InputNode;
  inputs: InputNode[];
  output: OutputNode;
  key: string;
  logic: AllLogicStatement;
  icon?: string;
}
